#!/usr/bin/php
<?php
function print_usage(){
    $file_name = basename(__FILE__);
	echo 'USAGE : '.$file_name." -f <file to network conf> -n <ifacel_label> -t [iface] -a [enable|disable]\n";
	echo '      : '.$file_name." -f <file to network conf> -n <ifacel_label> -t scan -a start -i <ip_addr_list> -s <scan>\n";
}

function print_help()
{
    $file_name = basename(__FILE__);
	print_usage();
	echo "\t".$file_name." : configure le réseau d'une unity\n";
	echo "\t\t[-f] 	| --file	 file to network conf\n";
	echo "\t\t[-n] 	| --name	 si le type=other indique le nom de l'interface\n";
	echo "\t\t[-t] 	| --type	 spécifie le type d'operation [iface|scan]\n";
    echo "\t\t[-a] 	| --action	 [enable|disable|start]\n";
	echo "\t\t[-i] 	| --ip	 	 ipv4 or ipv6 addresses\n";
	echo "\t\t[-s] 	| --scan	 ID or name for filetemp path into /tmp/docker directory\n";
    echo "\t\t[-D] 	| --debug    enable or disable debug mode\n";
    echo "\t\t[-h] 	| --help     print help\n";
}

$list_type = array('network','iface','scan');
$list_action = array('enable','disable','start');

//Définition des paramètres d'exécution
$longopts  = array("file:","name:","type:","action:","ip:","scan:","debug","help");
$options = getopt("f:n:t:a:i:s:Dh",$longopts);

if( getParam($options, 'h', 'help'))
{
    print_help();
}

$o_filepath = getParam($options, 'f', 'file',NULL);
$o_type = getParam($options, 't', 'type');
$o_name = getParam($options, 'n', 'name');
$o_action = getParam($options, 'a', 'action',"");
$o_adresses = getParam($options, 'i', 'ipaddr',NULL);
$o_scanid = getParam($options, 's', 'scan',NULL);
$o_debug = getParam($options, 'D', 'debug');

Trace($o_debug,"Debug Enabled");

if( empty($o_filepath) )
{
    echo "Missing required parameter -f, --file\n";
    print_usage();
    exit(1);
}

if( empty($o_name) )
{
    echo "Missing required parameter -n, --name\n";
    print_usage();
    exit(1);
}

if( !in_array($o_type,$list_type))
{
    $imp = implode(',',$list_type);
    echo "-t, --type parameter must be one on this ($imp)\n";
    print_usage();
    exit(1);
}

if( !in_array($o_action,$list_action) )
{
    $imp = implode(',',$list_action);
    echo "-a, --action parameter must be one on this ($imp)\n";
    print_usage();
    exit(1);
}

Trace($o_debug,"Opening configuration file $o_filepath");
$conf_file = @file_get_contents($o_filepath);
if( $conf_file != FALSE )
{
    $conf_file = @json_decode($conf_file,TRUE);
}

if( !is_array($conf_file) )
{
    echo "Unable to read and parse $o_filepath :" . error_get_last()."\n";
    exit(1);
}

Trace($o_debug,"Iface configuration searching $o_name");
$iface_conf = NULL;
foreach( $conf_file as $index => $row)
{
    if( $row['label'] == $o_name || $row['name'] == $o_name )
    {
        $iface_conf = $row;
        break;
    }
}

if( $iface_conf == NULL )
{
    echo "Unable to found iface $o_name into file $o_file_path\n";
    exit(1);
}

if( $o_type == 'iface' )
{
       if( $iface_conf['type'] == 'MGMT' && $iface_conf['label'] == 'MGMT' )
        {
               echo "MGMT IFACE, can't not be enabled or disabled. IGNORE THIS\n";
		exit(0);
        }

    if( $o_action == 'enable' )
    {
        Trace($o_debug,"Enabling interface");
        $docker_net = get_docker_network($o_debug,$iface_conf['label']);
        //Interface renomée ??? On search et remove l'ancienne si on la trouve
        if( !is_array($docker_net ) )
        {
            Trace($o_debug,"Docker network not found.");
            $docker_net = search_docker_network($o_debug,$iface_conf['name']);
            if( is_array($docker_net))
            {
                Trace($o_debug,"Iface set for another docker network. Cause is renaming network. Removing old network");
                rm_docker_network($o_debug,$docker_net);
                $docker_net = NULL;
            }
        }
        else
        {
            Trace($o_debug,"Docker network found. Checking network conf");
            //On vérifie si elle est bien configurée
            $keys = array('vlan','dhcp','netmask','gateway');
            foreach($keys as $key)
            {
                Trace($o_debug,"$key : iface_conf = ".$iface_conf[$key]." | docker_net = ".$docker_net[$key]);
                if($iface_conf[$key] != $docker_net[$key] )
                {
                    Trace($o_debug,"Docker network conf changed. Removing old docker network");
                    rm_docker_network($o_debug,$docker_net);
                    $docker_net = NULL;
                    break;
                }
            }
        }

        //Pour le driver ipvlan, il est nécessaire de faire le up de l'interface avant création
        $cmd = 'ifconfig '.$iface_conf['name'].' up';
        Trace($o_debug,"RUN EXEC : '$cmd'");
        exec($cmd,$output,$ret);

        //Pas de network, on le crée
        if( $docker_net == NULL )
        {
            Trace($o_debug,"Docker network not found or deleted. Creating new docker network");
            $ret = create_docker_network($o_debug,$iface_conf);
            if( $ret != 0 )
            {
                $cmd = 'ifconfig '.$iface_conf['name'].' down';
                Trace($o_debug,"RUN EXEC : '$cmd'");
                exec($cmd,$output,$ret);
                exit($ret);
            }
        }
         echo "Network iface enabled\n";
    }
    elseif( $o_action == 'disable' )
    {
        $cmd = 'ifconfig '.$iface_conf['name'].' down';
        Trace($o_debug,"RUN EXEC : '$cmd'");
        exec($cmd,$output,$ret);
        echo "Network iface disabled\n";
    }
	else
	{
		echo "Action must be only enable or disable for iface\n";
		print_usage();
		exit(1);
	}
}
elseif( $o_type == 'scan')
{
    if( $o_action == 'start' )
	{
		if( empty($o_adresses) )
		{
			echo "Missing required parameter -i, --ipaddr for scan->start\n";
			print_usage();
			exit(1);
		}
		if( empty($o_scanid) )
		{
			echo "Missing required parameter -s, --scan for scan->start\n";
			print_usage();
			exit(1);
		}
        echo "Starting network scan\n";
		docker_run_scan($o_debug,$iface_conf,$o_adresses,$o_scanid);
        echo "Network scan end\n";
	}
	else
	{
		echo "Action must be only start for scan\n";
		print_usage();
		exit(1);
	}
}

function docker_run_scan($o_debug,$iface_conf,$addr_to_scan,$scanid)
{
	$is_MGMT = FALSE;
	if( $iface_conf['type'] == 'MGMT' && $iface_conf['label'] == 'MGMT' )
	{
		$is_MGMT = TRUE;
	}

	$addr_to_scan = str_replace(','," ", $addr_to_scan);
	$container_network = $iface_conf['label'];
	$container_volume = $container_network.'_'.$scanid;
	$container_addr = $iface_conf['ip_addr'];
	$nmap_iface = $iface_conf['name'];
	$tmpfile = "/tmp/$scanid";

	if( $is_MGMT )
	{
		$tmpfile = "/tmp/docker/$scanid";
		$nmap_parameters = "-sV -oX $tmpfile $addr_to_scan";
	}
	else
	{
		$nmap_parameters = "-sV -oX $tmpfile $addr_to_scan";
	}

    if( !is_writable('/tmp/docker') )
    {
        $cmd = 'mkdir -p /tmp/docker/ ; chown root:root /tmp/docker ; chmod 700 /tmp/docker';
        Trace($o_debug,"RUN EXEC : '$cmd'");
	    exec($cmd);
    }

	$volume_parameter = '--mount type=bind,source="/tmp/docker",target="/tmp"';

    if( $is_MGMT )
    {
	$cmd = "/usr/bin/nmap $nmap_parameters";
    }
    elseif( $iface_conf['dhcp'] )
    {
	    $cmd = "docker run $volume_parameter --cap-add=NET_ADMIN --net=$container_network --rm esia-slim-nmap /usr/bin/nmap_dhcp $nmap_parameters";
    }
    else
    {
        if( filter_var($container_addr, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) )
        {
            $cmd = "docker run $volume_parameter --net=$container_network --ip=$container_addr --rm esia-slim-nmap /usr/bin/nmap $nmap_parameters";
        }
        elseif( filter_var($container_addr, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) )
        {
            $cmd = "docker run $volume_parameter --net=$container_network --ip6=$container_addr --rm esia-slim-nmap /usr/bin/nmap $nmap_parameters";
        }
        else
        {
            echo "Incorrect ipv4 or ipv6 addresse : ".$iface_conf['ip_addr']."\n";
            exit(1);
        }
    }

	Trace($o_debug,"RUN EXEC : '$cmd'");
	exec("$cmd 2>&1",$output,$ret);

   	if( $ret == 1 )
    {
        echo "Unable to run docker nmap scan".$output[0]."\n";
        exit(1);
    }
}

function get_docker_network($o_debug,$iface_name)
{
    $cmd = "docker network inspect $iface_name";
    Trace($o_debug,"RUN EXEC : '$cmd'");
    exec("$cmd 2>&1",$output,$ret);
    if( $ret == 1 )
    {
        Trace($o_debug,"Network $iface_name not found");
        return NULL;
    }

    Trace($o_debug,"Network $iface_name found");

    $output = implode("\n",$output);
    $output = json_decode($output,true);
    $output = array_shift($output);

    $ret = array();
    $ret['name'] = $output['Options']['parent'];
    $ret['label'] = $iface_name;
    $ret['vlan'] = 0;
    $ret['dhcp'] = false;
    $ret['netmask'] = explode('/',$output['IPAM']['Config'][0]['Subnet'])[1];
    $ret['gateway'] = $output['IPAM']['Config'][0]['Gateway'];

    if( $output['Driver'] == 'ipvlan' )
    {
        $exp = explode('.',$output['Options']['parent']);
        $ret['name'] = $exp[0];
        $ret['vlan'] = $exp[1];
    }
    return $ret;
}

function search_docker_network($o_debug,$parent)
{
    $networks = list_docker_networks($o_debug);
    foreach($networks as $network)
    {
        $conf = get_docker_network($o_debug,$network);
        if( $conf['name'] == $parent )
        {
            Trace($o_debug,"Network found for iface $parent");
            return $conf;
        }
    }
    Trace($o_debug,"Network  not found for iface $parent");
    return NULL;
}

function list_docker_networks($o_debug)
{
    $cmd = 'docker network ls|tr -s " " " " |cut -d " " -f2,3|grep -v "ID NAME"';
    Trace($o_debug,"RUN EXEC : '$cmd'");
    exec("$cmd 2>&1",$output,$ret);
    $ret = array();
    foreach($output as $row)
    {
        $exp = explode(' ',$row);
        if( $exp[1] == 'macvlan' || $exp[1] == 'ipvlan' )
        {
            $ret[] = $exp[0];
        }
    }
    return $ret;
}

function create_docker_network($o_debug,$iface_conf)
{
    $cmd = "docker network create";
    $ifname=$iface_conf['name'];
    if( $iface_conf['vlan'] == 0 )
    {
        $cmd.= " -d macvlan";
    }
    else
    {
        $cmd.= " -d ipvlan";
        $ifname.='.'.$iface_conf['vlan'];
    }

    if( !$iface_conf['dhcp'] )
    {
        if( filter_var($iface_conf['ip_addr'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) )
        {
            $subnet = ipv4NetworkInformation($iface_conf['ip_addr'],$iface_conf['netmask']);
            $subnet = long2ip($subnet['network']).'/'.$iface_conf['netmask'];
            $gateway = $iface_conf['gateway'];
        }
        elseif( filter_var($iface_conf['ip_addr'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) )
        {
            $subnet = ipv6NetworkInformation($iface_conf['ip_addr'],$iface_conf['netmask']);
            $subnet = inet_ntop($subnet['network']).'/'.$iface_conf['netmask'];
            $gateway = $iface_conf['gateway'];
        }
        else
        {
            echo "Incorrect ipv4 or ipv6 addresse : ".$iface_conf['ip_addr']."\n";
            return 1;
        }
        $cmd.= " --subnet=$subnet --gateway=$gateway";
    }

    $cmd.= " -o parent=$ifname ".$iface_conf['label'];
    Trace($o_debug,"RUN EXEC : '$cmd'");
    exec("$cmd 2>&1",$output,$ret);
    if( $ret == 1 )
    {
        echo "Unable to add docker network ".$iface_conf['label'].' : '.$output[0]."\n";
        return 1;
    }
    return 0;
}

function rm_docker_network($o_debug,$docker)
{
    $iface_label = $docker['label'];
    $iface_parent = $docker['name'];
    $iface_vlan = $docker['vlan'];

    $cmd = "docker network rm $iface_label";
    Trace($o_debug,"RUN EXEC : '$cmd'");
    exec("$cmd 2>&1",$output,$ret);

    if( $iface_vlan != 0 )
    {
        $cmd = "ip link del $iface_parent.$iface_vlan";
        Trace($o_debug,"RUN EXEC : '$cmd'");
        exec("$cmd 2>&1",$output,$ret2);
    }

    if( $ret == 1 )
    {
        echo "Unable to remove docker network $iface_label : ".$output[0]."\n";
        exit(1);
    }
}

function getParam($options, $shortName, $slongName, $default = null)
{
    $param = $default;
    if (isset($options[$shortName]))
    {
        $param = $options[$shortName];
    }
    elseif (isset($options[$slongName]))
    {
        $param = $options[$slongName];
    }
    return $param;
}

function ipv4NetworkInformation ($ip_address, $netmask)
{
    if( is_numeric($netmask) ) //Cidr
    {
        $ip_nmask_long = 32 - $netmask;
        $ip_broadcast_invert = pow(2,$ip_nmask_long)-1;
        $ip_nmask_long = ip2long('255.255.255.255')-$ip_broadcast_invert;
    }
    else //netmask string
    {
        $ip_nmask_long = ip2long($netmask);
        $ip_broadcast_invert = ip2long('255.255.255.255')-$ip_nmask_long;
    }
    //convertion des IPv4 en décimal
    $ip_address_long = ip2long($ip_address);
    //On fait le & pour avoir l'ip du réseau en décimal
    $ip_net = $ip_address_long & $ip_nmask_long;
    //calcul de l'adresse de broadcast
    $ip_broadcast = $ip_address_long | $ip_broadcast_invert;
    return array('network'=>$ip_net,'broadcast'=> $ip_broadcast);
}

function ipv6NetworkInformation ($ip_address, $cidr)
{
    //convertion des IPv6 en binaire
    $binari6 = ipv6ToBin($ip_address);
	$hosts = (128 -  $cidr);
    $networks = 128 - $hosts;
	$_m = str_repeat('1', $networks).str_repeat('0', $hosts);
	$_hexMask = null;
	foreach(str_split($_m, 4) as $segment)
	{
    	$_hexMask .= base_convert($segment, 2, 16);
	}
	$netmask = substr(preg_replace('/([A-f0-9]{4})/', '$1:', $_hexMask), 0, -1);
	$binari_netmask = ipv6ToBin($netmask);


	$network = $binari6 & $binari_netmask;
    //calcul de l'adresse de broadcast
    $ip_broadcast_invert = ~$binari_netmask;
    $ip_broadcast = $network | $ip_broadcast_invert;

	return array('network'=>$network,'broadcast'=> $ip_broadcast);
}

function ipv6ToBin($ipaddress)
{
    $pton = @inet_pton($ipaddress);
    if (!$pton)
    {
    	return false;
    }
    $binari6 = current(unpack('a*',$pton));
	return  $binari6;
}

function Trace($debug,$string)
{
	if(isset($debug))
	{
		if(is_array($string))
			print_r($string);
		else
			echo "$string\n";
	}
}
