#!/usr/bin/php
<?php
define('DATA_DIR',"/tmp/gesa/egide");

define('SCRIPT_NAME',basename($_SERVER['SCRIPT_NAME']));
define('EXIT_OK',0);
define('EXIT_ERROR',1);

function print_help($exit=null){
	echo "Used to manage (ADD/GET/DELETE) egide scan.\n";
	echo "\n";
	print_usage();
	echo "\n";
	echo "Actions :\n";
	echo " -A (--add)          add a scan in the scanner. (-i specify ONE task and not a list).\n";
	echo " -L (--list)         Get a list of all scan. Can be filtered with a list of status.\n";
	echo " -G (--get)          Get all the scan details in the -i list.\n";
	echo " -P (--partial)	   Get piece of the scan details (use with -p and -l option)\n";
	echo " -D (--delete)       Delete all the IDs in the -i list.\n";
	echo "\n";
	echo "Options :\n";
	echo " -i (--id=LIST)      Comma separated ids list.\n";
	echo " -a (--addr=LIST)    Comma separated address list (10.7.0.1,10.7.0.2,10.7.0.3 or 10.7.0.1-10.7.0.3 or 10.7.0.0/24).\n";
	echo " -e (--iface=IFACE)  Scan ethernet IFACE\n";
	echo " -o (--opts=OPTIONS) Options needed for scan. Null for ICMP. Community list for SNMP\n";
	echo " -s (--state=LIST)   1 or multiples scan status (pending,processing,terminated,error) comma separated.\n";
	echo " -p (--part=INT)     Piece of the result to get (use with -P)\n";
	echo " -l (--lenght=INT)   Size of the piece to get (use with -P)\n";
	echo "\n";
	echo " -h (--help)         Print help.\n";

	if($exit!=null) exit($exit);
}

function print_usage($exit=null){
	echo "Usage : \n";
	echo ' '.SCRIPT_NAME." -A [-i <ID>] -a <ADDR> -e <iface> [-o <OPTS>]\n";
	echo ' '.SCRIPT_NAME." -L [-s <STATUS>]\n";
	echo ' '.SCRIPT_NAME." -G -i <IDS>\n";
	echo ' '.SCRIPT_NAME." -D -i <IDS>\n";
	echo ' '.SCRIPT_NAME." -h\n";

	if($exit!=null) exit($exit);
}

function exit_error($msg,$print_usage=false){
	echo "ERROR : $msg\n";
	if($print_usage) print_usage(EXIT_ERROR);
	exit(EXIT_ERROR);
}

function getParam($options,$name1,$name2,$default=null){
	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function add_task($options){
	$id     = getParam($options,'i','id',time());
	$addr   = getParam($options,'a','addr');
	$iface   = getParam($options,'e','iface');
//	$opts   = getParam($options,'o','opts');

	if( $addr == null ) exit_error("You must specify addresses for the -A (--add) option",true);
    if( $iface == null ) exit_error("You must specify iface for the -A (--add) option",true);

	if( $id == null ) $id=time();
	else if( count(explode(',',$id))  > 1 ) exit_error("You must specify one ID for the -A (--add) option. Not a list",true);
	if( !is_numeric($id) ) exit_error("The id must be an integer.",true);

	if( is_file(DATA_DIR.'/'.$id))
	{
		exit_error("File already present, You must get and delete it first",FALSE);
	}

	$array["status"]="pending";
	$array["addresses"]=$options["a"];
	$array["iface"]=$iface;
//	if( $opts != null ) $array["options"]=$options["o"];

	$json=json_encode($array,JSON_PRETTY_PRINT);

    if( !is_dir(DATA_DIR))
    {
        mkdir(DATA_DIR);
    }
	file_put_contents(DATA_DIR.'/'.$id,$json."\n");
	echo "Task added\n";
}

function list_tasks($options){
	$states = getParam($options,'s','state');

	//On récupére la liste des fichiers
	$dir_items = @scandir(DATA_DIR,SCANDIR_SORT_ASCENDING);
	$files=array();
	foreach($dir_items as $item){
		$full_path = DATA_DIR."/$item";
		if(is_file($full_path)) $files[$item]=$full_path;
	}

	$ids=array();
	if( $states == null) $ids=array_keys($files);
	else{
		$states = strtolower($states);
		$states = explode(',',$states);
		foreach($files as $id => $full_path){
			$raw = @file_get_contents($full_path);
			$parsed = @json_decode($raw,true);

			$status = 'error';
			if( isset($parsed['status']) ) $status = $parsed['status'];

			foreach($states as $state){
				if( $status == $state ){
					array_push($ids,$id);
				}
			}
		}
	}

	echo implode(',',$ids)."\n";
}

function get_tasks($options){
	$ids = getParam($options,'i','id');
	if( $ids == null )  exit_error("You must specify one or serveral ids for the -G (--get) option",true);
	$ids = explode(',',$ids);

	$output = array();
	foreach($ids as $id){
		$full_path = DATA_DIR."/$id";
		$raw = @file_get_contents($full_path);
		$parsed = @json_decode($raw,true);
		if( isset($parsed["status"]) ) {
			$output[$id]=$parsed;
		}
	}
	echo json_encode($output,JSON_PRETTY_PRINT)."\n";
}

function get_partial($options){
        $ids = getParam($options,'i','id');
	$start = getParam($options,'p','part',0);
	$length = getParam($options,'l','length',1000);
        if( $ids == null )  exit_error("You must specify one or serveral ids for the -G (--get) option",true);
        $ids = explode(',',$ids);

        $output = array();
        foreach($ids as $id){
                $full_path = DATA_DIR."/$id";
                $raw = @file_get_contents($full_path);
                $parsed = @json_decode($raw,true);
                if( isset($parsed["status"]) ) {
                        $output[$id]=$parsed;
                }
        }
        echo substr(json_encode($output),$start,$length)."\n";
}

function remove_tasks($options){
	$ids = getParam($options,'i','id');
	if( $ids == null )  exit_error("You must specify one or several ids for the -D (--delete) option",true);
	$ids = explode(',',$ids);

	$deleted=0;
	$messages = array();
	foreach($ids as $id){
		//On vérifie si l'id est bien un int avant suppression. (On evite alors les ../..
		if( !is_numeric($id) ) {
			$messages[$id]="The id $id must be an integer";
			continue;
		}

		$full_path = DATA_DIR."/$id";

		//On vérifie ci le fichier supprimé est bien un fichier data de scan.
		$raw = @file_get_contents($full_path);
		$parsed = @json_decode($raw,true);
		if( !isset($parsed["status"]) ) {
			$messages[$id]="The file $full_path for task id $id can't be read";
			continue;
		}

		//Aprés avoir tester si on pouvait supprimer, on supprime.
		if(unlink($full_path)) $deleted++ ;
		else $messages[$id]="The script can't delete the file $fullpath for task id $id";
	}

	echo json_encode(array("total"=>count($ids),"deleted"=>$deleted,"errors"=>$messages),JSON_PRETTY_PRINT);
}

$longopts = array(
		'add','list','get','partial','delete','help',
		'id:','addr:','iface:','opts:','state:','part:','length'
);
$shortopts = 'ALGPDhi:a:e:o:s:p:l:';

$options = getopt($shortopts,$longopts);
if(      isset($options['h'])   || isset($options['help']))     print_help(EXIT_OK);
else if( isset($options['A'])   || isset($options['add']))      add_task($options);
else if( isset($options['L'])   || isset($options['list']))     list_tasks($options);
else if( isset($options['G'])   || isset($options['get']))      get_tasks($options);
else if( isset($options['P'])	|| isset($options['partial']))  get_partial($options);
else if( isset($options['D'])   || isset($options['delete']))   remove_tasks($options);
else exit_error('You must specify action.',true);
exit(EXIT_OK);
?>
